﻿Imports System.IO
Imports System.ComponentModel
Imports System.Collections.Generic

Public Class MailBox

    ' メール情報を格納するためのプロパティを定義
    Private Mailbox_Path As String
    Private _FileName As String
    Private _From As String
    Private _Subject As String
    Private _Dates As String
    Private _Body As String
    Private _Attach As String

    ' メールファイル名
    Public Property FileName() As String
        Get
            Return Me._FileName
        End Get
        Set(ByVal value As String)
            Me._FileName = value
        End Set
    End Property

    ' MailFrom
    Public Property From() As String
        Get
            Return Me._From
        End Get
        Set(ByVal value As String)
            Me._From = value
        End Set
    End Property

    ' 件名
    Public Property Subject() As String
        Get
            Return Me._Subject
        End Get
        Set(ByVal value As String)
            Me._Subject = value
        End Set
    End Property

    ' Dateヘッダ
    Public Property Dates() As String
        Get
            Return Me._Dates
        End Get
        Set(ByVal value As String)
            Me._Dates = value
        End Set
    End Property

    ' 本文
    Public Property Body() As String
        Get
            Return Me._Body
        End Get
        Set(ByVal value As String)
            Me._Body = value
        End Set
    End Property

    ' 添付ファイル数
    Public Property Attach() As String
        Get
            Return Me._Attach
        End Get
        Set(ByVal value As String)
            Me._Attach = value
        End Set
    End Property

    ' メールボックスのメールファイル一覧を取得するSELECTメソッド
    <DataObjectMethod(DataObjectMethodType.Select, True)> _
    Public Function GetMailbox(ByVal Name As String, ByVal Searchword As String, _
         ByVal SortExpression As String, ByVal SortDirection As String) As List(Of MailBox)

        Dim list As New List(Of MailBox)()
        Mailbox_Path = Name
        Dim files As String() = Directory.GetFiles( _
            Mailbox_Path, "*", SearchOption.AllDirectories)
        Dim mailfile As String
        Dim bobj As BASP21PROLib.basp21p = New BASP21PROLib.basp21p ' BASP21 Pro インスタンス
        Dim web As System.Web.HttpContext = System.Web.HttpContext.Current ' Session変数アクセス

        For Each fname As String In files ' メールボックス中のメールファイル処理
            mailfile = Path.GetFileName(fname)
            Dim ar As Object
            If web.Session(mailfile) Is Nothing Then
                ar = bobj.ReadMail(fname, "subject:date:from:body:nofile:", "")
                web.Session(mailfile) = ar  ' SessionにReadMailメソッド結果保存
            Else
                ar = web.Session(mailfile) ' 保存済みのReadMailメソッド結果取得
            End If

            Dim ok As Boolean = True
            If Searchword <> "" Then ' 全文検索
                ok = False
                For Each strdata As String In ar
                    If InStr(strdata, Searchword) > 0 Then
                        ok = True
                        Exit For
                    End If
                Next
            End If
            If ok Then
                Dim f As New MailBox
                f.FileName = mailfile  ' メールファイル名
                f.From = Mid(ar(2), 7) ' "From: "
                f.Subject = Mid(ar(0), 9) ' Subject:
                f.Dates = Mid(ar(1), 8) ' Date: 
                Dim attachfiles As String = ""
                Dim attachfilesCount As Integer = 0
                f.Body = body_trim(Mid(ar(3), 7)) ' 本文
                For Each contents As String In ar
                    If InStr(contents, "File: ") = 1 Then
                        attachfiles = attachfiles & "," & Mid(contents, 7)
                        attachfilesCount = attachfilesCount + 1
                    End If
                Next
                f.Attach = "" ' 添付ファイル数
                If attachfilesCount > 0 Then
                    'f.Attach = Mid(attachfiles, 2)
                    f.Attach = attachfilesCount
                End If
                list.Add(f)
            End If
        Next
        web.Session("totalcount") = list.Count
        ' ソートして返す
        Return sort_mailbox(list, SortExpression, SortDirection)
    End Function


    ' 指定されたメールファイルを削除するためのDELETEメソッド
    <DataObjectMethod(DataObjectMethodType.Delete, True)> _
    Public Sub Delete(ByVal name As String)
        ' メールファイルを削除
        Try
            File.Delete(name)
        Catch ex As IOException
            Throw New ApplicationException("ファイル削除失敗 " & name & ex.Message)
        End Try

    End Sub

    Const adInteger As Integer = 3, adDate As Integer = 7, adFldIsNullable As Integer = 32, adVarChar As Integer = 200
    ' ADOでソート  [参照の追加] Microsoft ActiveX Data Objects Recordset 6.0 Library
    Function sort_mailbox(ByVal ar As List(Of MailBox), ByVal SortExpression As String, _
                          ByVal SortDirection As String) As List(Of MailBox)
        If ar.Count = 0 Then Return ar
        Dim SortDirectionCommand As String
        Dim list As New List(Of MailBox)()
        Dim r As ADOR.Recordset = New ADOR.Recordset
        If SortExpression = "" Then SortExpression = "Dates"
        If SortDirection = "" Then SortDirection = "DESC"
        SortDirectionCommand = SortExpression & " " & SortDirection

        r.Fields.Append("Name", adVarChar, 128, adFldIsNullable)
        r.Fields.Append("From", adVarChar, 128, adFldIsNullable)
        r.Fields.Append("Subject", adVarChar, 256, adFldIsNullable)
        r.Fields.Append("Dates", adDate, , adFldIsNullable)
        r.Fields.Append("Body", adVarChar, 4096, adFldIsNullable)
        r.Fields.Append("Attach", adVarChar, 1024, adFldIsNullable)
        r.Open()
        For Each items As MailBox In ar
            r.AddNew()
            r.Fields("Name").Value = items.FileName
            r.Fields("From").Value = items.From
            r.Fields("Subject").Value = items.Subject
            r.Fields("Dates").Value = items.Dates
            r.Fields("body").Value = items.Body
            r.Fields("Attach").Value = items.Attach
            r.Update()
        Next
        r.Sort = SortDirectionCommand
        r.MoveFirst()
        Do Until r.EOF
            Dim f As New MailBox
            f.FileName = r.Fields("Name").Value
            f.From = r.Fields("From").Value
            f.Subject = r.Fields("Subject").Value
            f.Dates = format_date(r.Fields("Dates").Value)
            f.Body = r.Fields("Body").Value
            f.Attach = r.Fields("Attach").Value
            list.Add(f)
            r.MoveNext()
        Loop
        r.Close()
        r = Nothing
        Return list
    End Function

    Function format_date(ByVal dtstr As String) As String
        Dim dt As DateTime = dtstr
        Return dt.ToString("yyyy/MM/dd HH:mm")
    End Function

    Function body_trim(ByVal body As String) As String
        If Len(body) < 100 Then Return body
        Return Mid(body, 1, 100)
    End Function

End Class
